GOOGLE_URL = "google"
GOOGLE_ADS = "google_ads"
GOOGLE_HOTELS = "google_hotels"
GOOGLE_SEARCH = "google_search"
GOOGLE_IMAGES = "google_images"
GOOGLE_SUGGESTIONS = "google_suggest"
GOOGLE_TRAVEL_HOTELS = "google_travel_hotels"
GOOGLE_TRENDS_EXPLORE = "google_trends_explore"

BING_URL = "bing"
BING_SEARCH = "bing_search"

YANDEX_URL = "yandex"
YANDEX_SEARCH = "yandex_search"

BAIDU_URL = "baidu"
BAIDU_SEARCH = "baidu_search"

GOOGLE_SHOPPING_URL = "google_shopping"
GOOGLE_SHOPPING_SEARCH = "google_shopping_search"
GOOGLE_SHOPPING_PRODUCT = "google_shopping_product"
GOOGLE_SHOPPING_PRICING = "google_shopping_pricing"

WAYFAIR = "wayfair"
WAYFAIR_SEARCH = "wayfair_search"

UNIVERSAL = "universal_ecommerce"

AMAZON_URL = "amazon"
AMAZON_SEARCH = "amazon_search"
AMAZON_PRODUCT = "amazon_product"
AMAZON_PRICING = "amazon_pricing"
AMAZON_REVIEWS = "amazon_reviews"
AMAZON_QUESTIONS = "amazon_questions"
AMAZON_BEST_SELLERS = "amazon_bestsellers"
AMAZON_SELLERS = "amazon_sellers"
