SYNC_BASE_URL = "https://realtime.oxylabs.io/v1/queries"
ASYNC_BASE_URL = "https://data.oxylabs.io/v1/queries"

PROXY_BASE_URL = "realtime.oxylabs.io"
PROXY_PORT = 60000
NON_UNIVERSAL_DOMAINS = {"google", "bing", "amazon", "wayfair"}


DEFAULT_REQUEST_TIMEOUT = 165
DEFAULT_POLL_INTERVAL = 5
DEFAULT_REQUEST_TIMEOUT_ASYNC = 105
DEFAULT_JOB_COMPLETION_TIMEOUT = 50
