MOBILE = "mobile"
TABLET = "tablet"
DESKTOP = "desktop"
MOBILE_IOS = "mobile_ios"
TABLET_IOS = "tablet_ios"
DESKTOP_EDGE = "desktop_edge"
DESKTOP_OPERA = "desktop_opera"
DESKTOP_SAFARI = "desktop_safari"
MOBILE_ANDROID = "mobile_android"
DESKTOP_CHROME = "desktop_chrome"
TABLET_ANDROID = "tablet_android"
DESKTOP_FIREFOX = "desktop_firefox"
