RU = "ru"
UA = "ua"
TR = "tr"
CN = "cn"
COM_AI = "com.ai"
COM_PR = "com.pr"
SR = "sr"
ML = "ml"
COM_LB = "com.lb"
BF = "bf"
FM = "fm"
COM_MX = "com.mx"
BJ = "bj"
EE = "ee"
MV = "mv"
NE = "ne"
AT = "at"
GG = "gg"
AE = "ae"
CO_UZ = "co.uz"
AM = "am"
COM_SA = "com.sa"
TL = "tl"
COM_NA = "com.na"
COM_BH = "com.bh"
DK = "dk"
COM_SB = "com.sb"
RO = "ro"
BY = "by"
COM_CO = "com.co"
COM_GI = "com.gi"
CO_ID = "co.id"
MS = "ms"
COM_NG = "com.ng"
IS = "is"
COM_EG = "com.eg"
COM_ET = "com.et"
COM_AF = "com.af"
CH = "ch"
CO_AO = "co.ao"
CL = "cl"
CO_ZA = "co.za"
COM_NF = "com.nf"
DK_RO = "ro"
MD = "md"
ES = "es"
BJ_YO = "bj"
HU = "hu"
DJ = "dj"
COM_MT = "com.mt"
COM_EC = "com.ec"
CO_IN = "co.in"
LK = "lk"
CO_KE = "co.ke"
GY = "gy"
BE = "be"
VG = "vg"
CO_BW = "co.bw"
COM_VN = "com.vn"
CO_TZ = "co.tz"
NE_HA = "ne"
CO_ZW = "co.zw"
TO = "to"
KZ = "kz"
COM_UY = "com.uy"
IQ = "iq"
COM_TW = "com.tw"
RW = "rw"
AD = "ad"
COM_LY = "com.ly"
AL = "al"
CO_IL = "co.il"
KI = "ki"
COM = "com"
MU = "mu"
SC = "sc"
COM_HK = "com.hk"
COM_PA = "com.pa"
CA = "ca"
GE = "ge"
COM_GT = "com.gt"
LI = "li"
COM_KH = "com.kh"
CO_CR = "co.cr"
COM_BO = "com.bo"
CO_VE = "co.ve"
COM_NI = "com.ni"
TD = "td"
CF = "cf"
TK = "tk"
BI = "bi"
MG = "mg"
COM_BD = "com.bd"
COM_BZ = "com.bz"
GM = "gm"
LA = "la"
COM_KW = "com.kw"
CM = "cm"
HT = "ht"
NO = "no"
COM_FJ = "com.fj"
TM = "tm"
COM_SL = "com.sl"
COM_MM = "com.mm"
IM = "im"
SI = "si"
COM_QA = "com.qa"
COM_PE = "com.pe"
CD = "cd"
TT = "tt"
COM_TR = "com.tr"
TG = "tg"
CO_LS = "co.ls"
GR = "gr"
GL = "gl"
MK = "mk"
CO_ZM = "co.zm"
COM_PH = "com.ph"
IT = "it"
CO_JP = "co.jp"
WS = "ws"
COM_AR = "com.ar"
CO_MZ = "co.mz"
AZ = "az"
CO_CK = "co.ck"
FI = "fi"
COM_BN = "com.bn"
PT = "pt"
COM_TJ = "com.tj"
COM_CY = "com.cy"
CV = "cv"
COM_MY = "com.my"
IE = "ie"
COM_SG = "com.sg"
DE = "de"
BA = "ba"
LU = "lu"
BG = "bg"
CO_VI = "co.vi"
COM_OM = "com.om"
AS = "as"
DZ = "dz"
FR = "fr"
LV = "lv"
LT = "lt"
PS = "ps"
SE = "se"
CG = "cg"
NR = "nr"
CO_UG = "co.ug"
COM_VC = "com.vc"
JO = "jo"
CO_TH = "co.th"
RS = "rs"
BS = "bs"
COM_PK = "com.pk"
CO_UK = "co.uk"
SO = "so"
GA = "ga"
COM_UA = "com.ua"
HR = "hr"
COM_CU = "com.cu"
SK = "sk"
COM_NP = "com.np"
NU = "nu"
MN = "mn"
VU = "vu"
NL = "nl"
PT_ST = "st"
COM_BR = "com.br"
TH = "co.th"
MW = "mw"
COM_PG = "com.pg"
PL = "pl"
CO_NZ = "co.nz"
KG = "kg"
CI = "ci"
SH = "sh"
COM_DO = "com.do"
SN = "sn"
COM_JM = "com.jm"
CO_MA = "co.ma"
COM_TN = "com.tn"
DM = "dm"
COM_SV = "com.sv"
COM_SG_2 = "com.sg"
GP = "gp"
ME = "me"
COM_AG = "com.ag"
CZ = "cz"
COM_PY = "com.py"
MR_IN = "co.in"
COM_GH = "com.gh"
ST_LS = "co.ls"
BT = "bt"
RU_KZ = "kz"
IT_SM = "sm"
JE = "je"
TN = "tn"
COM_AU = "com.au"
ME_ME = "me"
PN = "pn"
HN = "hn"
CO_KR = "co.kr"
AR = "com.ar"
BO = "com.bo"
BZ = "com.bz"
UY = "com.uy"
COM_VE = "com.ve"
ID_TL = "tl"
