ELEMENT_TEXT = "element_text"
XPATH = "xpath"
XPATH_ONE = "xpath_one"
CSS = "css"
CSS_ONE = "css_one"
AMOUNT_FROM_STRING = "amount_from_string"
AMOUNT_RANGE_FROM_STRING = "amount_range_from_string"
JOIN = "join"
REGEX_FIND_ALL = "regex_find_all"
REGEX_SEARCH = "regex_search"
REGEX_SUBSTRING = "regex_substring"
LENGTH = "length"
SELECT_NTH = "select_nth"
CONVERT_TO_FLOAT = "convert_to_float"
CONVERT_TO_INT = "convert_to_int"
CONVERT_TO_STR = "convert_to_str"
AVERAGE = "average"
MAX = "max"
MIN = "min"
PRODUCT = "product"
